#ifndef DISPLAY_H_
#define DISPLAY_H_

#include "DisplayFile.h"
#include "Polyline.h"
#include "Window.h"
#include "xml/tinyxml.h"

class Display {
public:
	Display(const Window& window, const Coordinate& planeNormal, const Coordinate& viewUp);
	virtual ~Display();

	void add(GeometricElement* element);

	//allows iteration over display file elements
	DisplayFile::Iterator begin();
	DisplayFile::Iterator end();

	//controls the elements
	void translate(const std::string& elementName, double xOffset, double yOffset);
	void scale(const std::string& elementName, double xScale, double yScale, double zScale);
	void rotate(const std::string& elementName, MatrixFactory::Axis axis,  double angle, const Coordinate& reference);
	void rotate(const std::string& elementName, MatrixFactory::Axis axis, double angle);

	const GeometricElement* getElement(const std::string& name);

	//controls the window
	void translateWindow(double xOffset, double yOffset, double zOffset);
	void scaleWindow(double xRatio, double yRatio, double zRatio);
	void rotateWindow(MatrixFactory::Axis axis, double angle);

	void drawOn(Viewport& viewport);

	void save(const std::string& filepath);
	static Display* load(const std::string& filepath);
protected:
	void rotateWindowOrientation(MatrixFactory::Axis axis, double angle);
private:
	DisplayFile displayFile;
	DisplayFile ppcFile;

	Point planeNormal, viewUp;

	//window orientation and inverse window orientation
	Matrix<double>* wOrientation;
	Matrix<double>* iwOrientation;

	Window window;

	double tAngle;
};

#endif /* DISPLAY_H_ */
