#ifndef GEOMETRICELEMENT_H_
#define GEOMETRICELEMENT_H_

#include "Matrix.h"
#include "MatrixFactory.h"
#include "DrawingConfig.h"
#include "Vector.h"
#include "xml/tinyxml.h"
#include <memory>
#include <list>

class Rectangle;
class GeometricElement;
typedef std::list<GeometricElement*> ElementsList;

class GeometricElement{
public:
	enum ElementType { UNSPECIFIED, POINT, LINE, POLYLINE, POLYGON, FILLED_POLYGON, BEZIER, BSPLINE, ELEMENT3D, SURFACE };
	static const std::string ELEMENT_TYPES_STR[];

	GeometricElement(unsigned nPoints = 1, GeometricElement::ElementType type = UNSPECIFIED, const std::string& name = "", const DrawCfg& drawingConfig = DrawCfg());
	GeometricElement(const GeometricElement& other);

	virtual GeometricElement* clone() const = 0;
	virtual ElementsList clip(const Rectangle& clippingArea) const;
	virtual ElementsList clip3d(const Rectangle& clippingArea, const Coordinate& projectionCenter) const;

	bool is(ElementType type) const;
	ElementType getType() const;

	bool operator==(const GeometricElement& other) const;
	bool operator!=(const GeometricElement& other) const;

	void transform(const Matrix<double>& transformationMatrix);

	inline double operator()(unsigned coordinateNumber, unsigned pointNumber) const{
		return (*matrix)(coordinateNumber,pointNumber);
	}

	inline unsigned nPoints() const { return (*matrix).getColumns(); }

	void translate(double xOffset, double yOffset, double zOffset);
	void scale(double xScale, double yScale, double zScale, const Coordinate& reference);
	void scale(double xScale, double yScale, double zScale);
	void rotate(MatrixFactory::Axis axis, double angle, const Coordinate& reference);
	void rotate(MatrixFactory::Axis axis, double angle);

	virtual void project(const Rectangle& planeRectangle, const Coordinate& projectionCenter);

	std::string getName() const;
	Coordinate getCenter() const;
	DrawCfg getDrawingConfig() const;

	virtual std::string toString() const;
	virtual TiXmlElement* toXmlElement() const = 0;

	void save(const std::string& filepath) const;
	static GeometricElement* load(const std::string& filepath);

	virtual ~GeometricElement();
protected:
	void recalculateCenter();

	//matrix that represents the points
	Matrix<double>* matrix;
	Coordinate center;

	std::string name;
	ElementType type;
	DrawCfg drawingConfig;
};

#endif /* GEOMETRICELEMENT_H_ */
