#ifndef POLYGON_H_
#define POLYGON_H_

#include <memory>
#include "Polyline.h"
#include "Vector.h"

class Polygon : public Polyline{
public:
	Polygon(CoordinateList& points, const std::string& name, bool filled, const DrawCfg& drawCfg = DrawCfg());
	Polygon(const Polygon& other);

	virtual GeometricElement* clone() const;
	virtual ElementsList clip(const Rectangle& clippingArea) const;
	ElementsList clipFilled(const Rectangle& clippingArea) const;

	bool isClockwise() const;

	virtual TiXmlElement* toXmlElement() const;

	virtual ~Polygon();

	static Polygon* createFromXml(TiXmlElement* element);
protected:
	virtual LineList getLines() const;

	typedef std::vector<std::pair<bool, Coordinate> > IntersectionList;
	void waGetIntersectionPair(const Rectangle& clippingArea, IntersectionList& subj, IntersectionList& clip) const;
	void waFindEnteringIntersections(IntersectionList& subj, const Rectangle& clippingArea, CoordinateList& enteringIntersections) const;
	IntersectionList::iterator waFindNearest(IntersectionList& list, Coordinate coord) const;
	ElementsList waCutPolygons(IntersectionList& subj, IntersectionList& clip, CoordinateList& enteringIntersections) const;

	CoordinateList getCoordinatesCW() const;

	Coordinate first() const;
	Coordinate last() const;
};

class Rectangle : public Polygon {
public:
	Rectangle(const Coordinate& min, const Coordinate& max, const std::string& name, bool filled, const DrawCfg& drawCfg = DrawCfg());

	Coordinate min() const;
	Coordinate max() const;

	double height() const;
	double width() const;

	static std::auto_ptr<CoordinateList> createCoordinates(const Coordinate& min, const Coordinate& max);

};

#endif /* POLYGON_H_ */
