#ifndef VECTOR_H_
#define VECTOR_H_

#include <sstream>
#include <cmath>

struct Vector {
	Vector(double x, double y, double z) : x(x), y(y), z(z) {}
	Vector(const double* coordinate) : x(coordinate[0]), y(coordinate[1]), z(coordinate[2])  {}
	Vector() {}

	inline double length() const { return sqrt(x*x + y*y); }

	inline double operator*(const Vector& other) const{
		return ( x * other.x ) + ( y * other.y ) ;
	}

	inline Vector operator+(const Vector& other) const{
		return Vector((x + other.x),(y + other.y),(z + other.z));
	}

	inline double angleXY(const Vector& other) const {
		return 180 * (atan2(other.y, other.x) - atan2(y, x))  / M_PI;
	}

	inline double angleXZ(const Vector& other) const {
		return 180 * (atan2(other.z, other.x) - atan2(z, x))  / M_PI;
	}

	inline double angleYZ(const Vector& other) const {
			return 180 * (atan2(other.z, other.y) - atan2(z, y))  / M_PI;
		}

	inline double distance(const Vector& other) const{
		return sqrt( pow(other.x - x, 2) + pow(other.y - y, 2) + pow(other.z - z, 2));
	}

	inline double smallestAngleXY(const Vector& other) const{
		return 180 * (acos( ( (*this) * other) / ( length() * other.length()) )) / M_PI;
	}

	double operator==(const Vector& other) const {
		return x == other.x && y == other.y && z == other.z;
	}

	std::string toString() const{
		std::stringstream buffer; buffer << "(" << x << ", " << y << ", " << z << ")";
		return buffer.str();
	}

	double x, y, z;
};

typedef Vector Coordinate;

#include <vector>
typedef std::vector<Coordinate> CoordinateList;

#endif /* VECTOR_H_ */
