#ifndef WINDOW_H_
#define WINDOW_H_

#include "DisplayFile.h"
#include "Line.h"
#include "Polygon.h"
#include "Bezier.h"

class Viewport;

class Window{
public:
	Window(const Coordinate& minCoord, const Coordinate& maxCoord, const Coordinate& projectionCenter);
	Window(const Window& other);

	void draw(DisplayFile& displayFile, Viewport& viewport);

	void translate(double xOffset, double yOffset, double zOffset);
	void scale(double xScale, double yScale, double zScale);

	Coordinate getCenter() const;
	Coordinate getProjectionCenter() const;

	Coordinate min() const;
	Coordinate max() const;

	virtual ~Window();
protected:
	static const double SUBCANVAS_RATIO = 0.95;

	void setViewportMatrix(const Viewport& viewport);

	DisplayFile* project(DisplayFile& displayFile);
	DisplayFile* clip(DisplayFile& displayFile);

	Rectangle wRectangle;
	Coordinate projectionCenter;

	//Viewport Transformation Matrix
	Matrix<double> vpTMatrix;
};

#endif /* WINDOW_H_ */
