#ifndef CREATEELEMENTWINDOW_H_
#define CREATEELEMENTWINDOW_H_

#include <gtkmm.h>
#include "Point.h"

class CreatePointWindow : public Gtk::Dialog {
public:
	CreatePointWindow(Gtk::Window& parent, bool askName = true);

	Coordinate getCoordinate();
	Point* getPoint();

	virtual ~CreatePointWindow();
private:
	void onResponseClicked(int response);
	void onColorClicked();

	Gtk::Label labelTypeCoordinates;

	Gtk::HBox boxCoordinates, boxName;
	Gtk::ColorButton btColor;
	Gtk::Label labelX,labelY,labelZ;
	Gtk::Entry entryX,entryY,entryZ, nameEntry;
	Gdk::Color color;

	Gtk::Frame frameCoordinates;

	Coordinate coordinate;
};


// ---- CreatePolylineWindow ----

#include "Polyline.h"

class CreatePolylineWindow : public Gtk::Dialog {
public:
	CreatePolylineWindow(Gtk::Window& parent);

	Polyline* getElement();

	virtual ~CreatePolylineWindow();
protected:
	void onCreatePointClicked();
	void onColorSet();

	CoordinateList coordinates;
	Gtk::Window& parent;

	Gtk::HBox nameHbox;
	Gtk::Label labelName;
	Gtk::Entry entryName;

	Gtk::Frame pointsFrame;
	Gtk::VBox pointsVBox;

	Gtk::Button buttonAddPoint;
	Gtk::Button buttonDeletePoint;

	class PointsColumns : public Gtk::TreeModel::ColumnRecord{
	public:
		PointsColumns(){
			add(x); add(y); add(z);
		}

		Gtk::TreeModelColumn<double> x;
		Gtk::TreeModelColumn<double> y;
		Gtk::TreeModelColumn<double> z;
	};

	PointsColumns pointColumn;
	Gtk::TreeView pointsView;
	Glib::RefPtr<Gtk::ListStore> pointsModel;
	Gtk::HBox pointsButtonsHBox;

	Gtk::ColorButton btColor;
	Gdk::Color color;
};

// ---- CreatePolygonWindow ----

#include "Polygon.h"

class CreatePolygonWindow : public CreatePolylineWindow {
public:
	CreatePolygonWindow(Window& window);

	Polygon* getElement();

	~CreatePolygonWindow();
protected:
	Gtk::CheckButton toggleFilled;
};


// ---- CreateBezierWindow ----

#include "Bezier.h"

class CreateBezierWindow : public CreatePolylineWindow {
public:
	CreateBezierWindow(Window& window);

	Bezier* getElement();

	~CreateBezierWindow();
protected:
	unsigned nPoints;
};

// ---- CreateBSPlineWindow ----

#include "BSpline.h"

class CreateBSplineWindow : public CreatePolylineWindow {
public:
	CreateBSplineWindow(Window& window);

	BSpline* getElement();

	~CreateBSplineWindow();
};

// ---- CreateBSurfaceWindow ----

#include "BicubicSurface.h"

class CreateBSurfaceWindow : public CreatePolylineWindow {
public:
	CreateBSurfaceWindow(Window& window);

	BicubicSurface* getElement();

	~CreateBSurfaceWindow();
};

#endif /* CREATEELEMENTWINDOW_H_ */
