#ifndef MAINWINDOW_H_
#define MAINWINDOW_H_

#include <gtkmm.h>

#include "DisplayFile.h"
#include "GeometricElement.h"
#include "Window.h"
#include "GSViewport.h"
#include "CreateElementWindow.h"

class MainWindow : public Gtk::Window {
public:
	MainWindow();
	virtual ~MainWindow();
private:
	void addMenuToolbar();

	void setViewControl();

	void refreshElementsList();
	void onCreatePolylineClicked();
	void onCreatePointClicked();
	void onCreatePolygonClicked();
	void onCreateBezierClicked();
	void onCreateBSplineClicked();
	void onCreateBSurfaceClicked();

	void on_save_object_clicked();
	void on_save_scene_clicked();

	void on_open_object_clicked();
	void on_open_scene_clicked();

	void changeScene(Display* newScene);

	void controlConstantChanged();
	void onUpClicked();
	void onDownClicked();
	void onLeftClicked();
	void onRightClicked();
	void onZoomInClicked();
	void onZoomOutClicked();
	void onRotateCWClicked();
	void onRotateCCWClicked();

	MatrixFactory::Axis getRotationAxis();

	void onChangeReferenceClicked();

	void refresh();

	//just creates some objects for testing
	void createTestElements();

	Display display;
	GSViewport viewport;

	//Widgets
	//the main container
	Gtk::VBox mainVBox;
	//separates the viewport frame from the object list frame
	Gtk::HBox mainHBox;

	Gtk::VBox objectsAndViewBox;

	//these two are used to create the menu
	Glib::RefPtr<Gtk::UIManager> uiManager;
	Glib::RefPtr<Gtk::ActionGroup> actionGroup;

	Gtk::Frame objectsFrame, viewControlFrame, viewportFrame, rotationAxisFrame;
	Gtk::VBox rotationAxisVBox;
	Gtk::HBox referenceHBox, rotationAxisHBox;
	Gtk::RadioButton radioAxisX, radioAxisY, radioAxisZ;
	Gtk::RadioButton radioReferenceCenter, radioReferenceOther;
	Coordinate transformationReference;
	Gtk::Statusbar statusBar;

	//view control frame itens
	Gtk::VBox viewControlBox;
	Gtk::HBox controlSelectionBox;
	Gtk::RadioButton radioControlViewport, radioControlObjects;
	double controlConstant;
	Gtk::Table viewControlTable;
	Gtk::Entry entryControlConstant;
	Gtk::Button btUp,btDown,btLeft,btRight,btZoomIn,btZoomOut,btRotateCW,btRotateCCW;
	Gtk::Image btUpImg,btDownImg,btLeftImg,btRightImg,btZoomInImg,btZoomOutImg,btRotateCWImg,btRotateCCWImg;

	class ElementColumn : public Gtk::TreeModel::ColumnRecord{
		public:
			ElementColumn(){
				add(elementName); add(elementType);
			}

			Gtk::TreeModelColumn<std::string> elementName;
			Gtk::TreeModelColumn<std::string> elementType;
		};

	ElementColumn elementColumn;
	Gtk::TreeView elementsView;
	Gtk::ScrolledWindow elementsViewScroll;
	Glib::RefPtr<Gtk::ListStore> elementsModel;
};

class ErrorDialog : public Gtk::Dialog{
public:
	ErrorDialog(Window& parent, const std::exception& e)
		: errorLabel(std::string(e.what()))
	{
		set_title("Error");
		set_border_width(5);

		add_button(Gtk::Stock::OK,Gtk::RESPONSE_OK);

		get_vbox()->pack_start(errorLabel,Gtk::PACK_EXPAND_WIDGET);

		set_transient_for(parent);

		show_all_children();
	}
protected:
	Gtk::Label errorLabel;
};

#endif /* MAINWINDOW_H_ */
