#include "Point.h"
#include "Polygon.h"

Point::Point(const Coordinate& coordinate, const std::string& name, const DrawCfg& drawCfg)
	: GeometricElement(1, GeometricElement::POINT, name,drawCfg)
{
	(*matrix)(0,0) = coordinate.x;
	(*matrix)(1,0) = coordinate.y;
	(*matrix)(2,0) = coordinate.z;
	(*matrix)(3,0) = 1;

	recalculateCenter();
}

Point::Point(const Point& other) : GeometricElement(other){}

Coordinate Point::getCoordinate() const {
	return Coordinate((*matrix)(0,0),(*matrix)(1,0),(*matrix)(2,0));
}

ElementsList Point::clip(const Rectangle& clippingArea) const{
	ElementsList list;

	Coordinate coordinate = getCoordinate();

	Coordinate clipMin = clippingArea.min();
	Coordinate clipMax = clippingArea.max();

	bool inHorizontalBounds = ( coordinate.x > clipMin.x ) && ( coordinate.x < clipMax.x ) ;
	bool inVerticalBounds = ( coordinate.y > clipMin.y ) && ( coordinate.y < clipMax.y ) ;

	if(inHorizontalBounds && inVerticalBounds)
		list.push_back(this->clone());

	return list;
}

GeometricElement* Point::clone() const{
	return new Point(*this);
}

TiXmlElement* Point::toXmlElement() const{
	TiXmlElement *pointElement = new TiXmlElement("ponto");

	pointElement->SetAttribute("nome",name.c_str());
	pointElement->SetAttribute("cor", drawingConfig.getColor().colorCode().c_str());
	pointElement->SetDoubleAttribute("x",(*matrix)(0,0));
	pointElement->SetDoubleAttribute("y",(*matrix)(1,0));
	pointElement->SetDoubleAttribute("z",(*matrix)(2,0));

	return pointElement;
}

Point* Point::createFromXml(TiXmlElement* element){
	if(!element)
		throw InvalidParameterException("invalid XML");

	Coordinate coord;
	element->QueryDoubleAttribute("x",&coord.x);
	element->QueryDoubleAttribute("y",&coord.y);
	element->QueryDoubleAttribute("z",&coord.z);

	std::string name(element->Attribute("nome"));
	DrawCfg drawingCfg(Color(std::string("#").append(element->Attribute("cor"))));

	return new Point(coord,name,drawingCfg);
}

Point::~Point() {
}
