#include "gui/CreateElementWindow.h"
#include <sstream>

// --- CreatePointWindow ----

CreatePointWindow::CreatePointWindow(Gtk::Window& parent, bool askName)
	: labelTypeCoordinates("Point Name: "), labelX("x:"),
	  labelY("y:"), labelZ("z:"),frameCoordinates("Coordinates"), coordinate(0,0,0)
{
	set_title("Create point");
	set_border_width(5);

	if(askName){
		get_vbox()->pack_start(boxName);
		boxName.pack_start(labelTypeCoordinates);
		boxName.pack_start(nameEntry);
		boxName.pack_start(btColor);

		btColor.signal_color_set().connect(sigc::mem_fun(*this, &CreatePointWindow::onColorClicked));
	}

	get_vbox()->pack_start(frameCoordinates);
	get_vbox()->set_spacing(5);

	frameCoordinates.add(boxCoordinates);

	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

	signal_response().connect(sigc::mem_fun(*this,&CreatePointWindow::onResponseClicked));

	entryX.set_width_chars(6);
	entryY.set_width_chars(6);
	entryZ.set_width_chars(6);
	boxCoordinates.pack_start(labelX,Gtk::PACK_EXPAND_PADDING);
	boxCoordinates.pack_start(entryX,Gtk::PACK_EXPAND_PADDING);
	boxCoordinates.pack_start(labelY,Gtk::PACK_EXPAND_PADDING);
	boxCoordinates.pack_start(entryY,Gtk::PACK_EXPAND_PADDING);
	boxCoordinates.pack_start(labelZ,Gtk::PACK_EXPAND_PADDING);
	boxCoordinates.pack_end(entryZ,Gtk::PACK_EXPAND_PADDING);
	boxCoordinates.set_spacing(5);
	boxCoordinates.set_border_width(5);

	set_transient_for(parent);

	show_all_children();
}

void CreatePointWindow::onColorClicked(){
	color = btColor.get_color();
}

void CreatePointWindow::onResponseClicked(int response){
	if(response == Gtk::RESPONSE_OK){
		double x,y,z;
		std::stringstream buffer;

		buffer << entryX.get_text().raw();
		buffer >> x;
		buffer.str("");
		buffer.clear();

		buffer << entryY.get_text().raw();
		buffer >> y;
		buffer.str("");
		buffer.clear();

		buffer << entryZ.get_text().raw();
		buffer >> z;
		buffer.str("");
		buffer.clear();

		coordinate = Coordinate(x,y,z);
	}
}

Point* CreatePointWindow::getPoint(){
	return new Point(coordinate, nameEntry.get_text(), DrawCfg(Color(color.to_string())));
}

Coordinate CreatePointWindow::getCoordinate(){
	return coordinate;
}

CreatePointWindow::~CreatePointWindow() {}

// ---- CreatePolylineWindow ----

CreatePolylineWindow::CreatePolylineWindow(Gtk::Window& parent)
	: parent(parent), labelName("Polyline name:"),
	  pointsFrame("Points:"),
	  buttonAddPoint(Gtk::Stock::NEW),
	  buttonDeletePoint(Gtk::Stock::DELETE)
{
	set_title("Create element");

	get_vbox()->pack_start(nameHbox,Gtk::PACK_SHRINK);
	get_vbox()->pack_end(pointsFrame,Gtk::PACK_EXPAND_WIDGET);

	nameHbox.pack_start(labelName);
	nameHbox.pack_start(entryName);
	nameHbox.pack_end(btColor);

	nameHbox.set_spacing(2);

	add_button(Gtk::Stock::OK,Gtk::RESPONSE_OK);
	add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);

	pointsFrame.add(pointsVBox);

	pointsModel = Gtk::ListStore::create(pointColumn);
	pointsView.set_model(pointsModel);

	pointsView.append_column("x",pointColumn.x);
	pointsView.append_column("y",pointColumn.y);
	pointsView.append_column("z",pointColumn.z);

	pointsView.set_size_request(0,200);

	pointsVBox.pack_start(pointsView,Gtk::PACK_EXPAND_WIDGET);
	pointsVBox.pack_start(pointsButtonsHBox,Gtk::PACK_SHRINK);

	pointsButtonsHBox.pack_start(buttonAddPoint,Gtk::PACK_EXPAND_WIDGET);
	pointsButtonsHBox.pack_end(buttonDeletePoint,Gtk::PACK_EXPAND_WIDGET);

	buttonAddPoint.signal_clicked().connect(sigc::mem_fun(*this,&CreatePolylineWindow::onCreatePointClicked));

	btColor.signal_color_set().connect(sigc::mem_fun(*this,&CreatePolylineWindow::onColorSet));

	set_transient_for(parent);

	show_all_children();
}

void CreatePolylineWindow::onCreatePointClicked(){
	CreatePointWindow createPointWindow(*this,false);
	int result = createPointWindow.run();

	if(result == Gtk::RESPONSE_OK){
		Coordinate coord = createPointWindow.getCoordinate();
		coordinates.push_back(coord);

		Gtk::TreeModel::Row row = *(pointsModel->append());
		row[pointColumn.x] = coord.x;
		row[pointColumn.y] = coord.y;
		row[pointColumn.z] = coord.z;
	}
}

void CreatePolylineWindow::onColorSet(){
	color = btColor.get_color();
}

Polyline* CreatePolylineWindow::getElement(){
	return new Polyline(coordinates, entryName.get_text(), DrawCfg(Color(color.to_string())));
}

CreatePolylineWindow::~CreatePolylineWindow() {}

// ---- CreatePolygonWindow ----

CreatePolygonWindow::CreatePolygonWindow(Window& window)
	: CreatePolylineWindow(window), toggleFilled("Filled"){
	nameHbox.pack_start(toggleFilled);

	set_title("Create Polygon");
	labelName.set_text("Polygon name:");

	show_all();
}

Polygon* CreatePolygonWindow::getElement(){
	return new Polygon(coordinates, entryName.get_text(), toggleFilled.get_active(), DrawCfg(Color(color.to_string())));
}


CreatePolygonWindow::~CreatePolygonWindow(){}

// ---- CreateBezierWindow ----

CreateBezierWindow::CreateBezierWindow(Window& window)
	: CreatePolylineWindow(window){
	set_title("Create Bezier Curve");
	labelName.set_text("Curve name:");

	show_all();
}

Bezier* CreateBezierWindow::getElement(){
	return new Bezier(coordinates, entryName.get_text(), 100, DrawCfg(Color(color.to_string())));
}

CreateBezierWindow::~CreateBezierWindow(){
}

// ---- CreateBSplineWindow ----

CreateBSplineWindow::CreateBSplineWindow(Window& window) : CreatePolylineWindow(window){
	set_title("Create BSpline Curve");
	labelName.set_text("Curve name:");

	show_all();
}

BSpline* CreateBSplineWindow::getElement(){
	return new BSpline(coordinates, entryName.get_text(), 100, DrawCfg(Color(color.to_string())));
}

CreateBSplineWindow::~CreateBSplineWindow(){
}

// ---- CreateBSurfaceWindow ----

CreateBSurfaceWindow::CreateBSurfaceWindow(Window& window) : CreatePolylineWindow(window){
	set_title("Create Bicubic surface");
	labelName.set_text("Surface name:");

	show_all();
}

BicubicSurface* CreateBSurfaceWindow::getElement(){
	return new BicubicSurface(coordinates, entryName.get_text(), 20, 20, DrawCfg(Color(color.to_string())));
}

CreateBSurfaceWindow::~CreateBSurfaceWindow(){
}

