#include "gui/GSViewport.h"
#include "cairomm-1.0/cairomm/enums.h"

GSViewport::GSViewport(Display& display) : Viewport(display){
	refreshDimensions();
}

void GSViewport::refreshDimensions(){
	width = get_allocation().get_width();
	height = get_allocation().get_height();
}

bool GSViewport::on_expose_event(GdkEventExpose* event){
	refreshDimensions();

	drawingArea = get_window();
	cairoContext = drawingArea->create_cairo_context();
	if(!drawingArea)
		return false;

	clipToEventRegion(event);

	cairoContext->set_source_rgb(1,1,1);
	cairoContext->paint();

	cairoContext->set_line_join(Cairo::LINE_JOIN_ROUND);
	cairoContext->set_line_cap(Cairo::LINE_CAP_ROUND);
	display.drawOn(*this);

	return true;
}

void GSViewport::clipToEventRegion(GdkEventExpose* event){
	clippingRect.x = event->area.x;
	clippingRect.y = event->area.y;
	clippingRect.width = event->area.width;
	clippingRect.height = event->area.height;

	cairoContext->rectangle(clippingRect.x, clippingRect.y, clippingRect.width, clippingRect.height);
	cairoContext->clip();
}

void GSViewport::redraw(){
	refreshDimensions();

	if(drawingArea)
		drawingArea->invalidate_rect(Gdk::Rectangle(0,0,width,height), true);
}

void GSViewport::draw(const GeometricElement& element){
	DrawCfg drawingConfig = element.getDrawingConfig();

	Gdk::Color color(drawingConfig.getColor().toString());
	cairoContext->set_source_rgb(color.get_red_p(),color.get_green_p(), color.get_blue_p());
	cairoContext->set_line_width(drawingConfig.getBorderWidth());

	cairoContext->move_to( element(0,0), height - element(1,0) );
	for(unsigned i = 0; i < element.nPoints(); i++)
		cairoContext->line_to( element(0,i), height - element(1,i) );

	if(element.is(GeometricElement::FILLED_POLYGON))
		cairoContext->fill();

	cairoContext->stroke();

}

GSViewport::~GSViewport() {
}
