#include "gui/MainWindow.h"

#include "Polygon.h"
#include "Line.h"
#include "Point.h"
#include "Bezier.h"
#include "Element3D.h"
#include "BSpline.h"
#include "BicubicSurface.h"

MainWindow::MainWindow() :
	display(::Window(Coordinate(-500,-500,0),Coordinate(500,500,0),Coordinate(0,0,0)),Coordinate(0,0,-1),Coordinate(0,1,0)),
	viewport(display),
	objectsFrame("Objects"),
	viewControlFrame("Control"),
	viewportFrame("Viewport"),
	rotationAxisFrame("Rotation"),
	radioAxisX("x"),
	radioAxisY("y"),
	radioAxisZ("z"),
	radioReferenceCenter("Center"),
	radioReferenceOther("Other..."),
	transformationReference(0,0,0),
	radioControlViewport("Viewport"), radioControlObjects("Objects"),
	controlConstant(10),
	viewControlTable(3,3,true)
{
	set_title("Basic Graphics System");

	add(mainVBox);

	addMenuToolbar();

	mainVBox.pack_start(mainHBox, Gtk::PACK_EXPAND_WIDGET);
	mainHBox.pack_start(objectsAndViewBox,Gtk::PACK_SHRINK);
	mainHBox.pack_end(viewportFrame,Gtk::PACK_EXPAND_PADDING);

	objectsAndViewBox.set_size_request(170,0);
	objectsAndViewBox.pack_start(objectsFrame,Gtk::PACK_EXPAND_WIDGET);
	objectsAndViewBox.pack_end(viewControlFrame,Gtk::PACK_SHRINK);

	setViewControl();

	//creates the elements list
	elementsModel = Gtk::ListStore::create(elementColumn);
	elementsView.set_model(elementsModel);
	elementsView.append_column("Type",elementColumn.elementType);
	elementsView.append_column("Name",elementColumn.elementName);

	objectsFrame.add(elementsViewScroll);
	elementsViewScroll.add(elementsView);
	elementsViewScroll.set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);

	viewportFrame.add(viewport);
	viewport.set_size_request(550,550);

	mainVBox.pack_end(statusBar,Gtk::PACK_SHRINK);

//	comment the line below to hide the test elements
//	createTestElements();
//	refresh();

	show_all_children();
}

void MainWindow::refreshElementsList(){
	elementsModel->clear();

	DisplayFile::Iterator it = display.begin();
	while(it != display.end()){
		const GeometricElement* element = (*it);

		Gtk::TreeModel::Row row = *(elementsModel->append());
		row[elementColumn.elementName] = element->getName();
		row[elementColumn.elementType] = GeometricElement::ELEMENT_TYPES_STR[element->getType()];

		it++;
	}
}

void MainWindow::setViewControl(){
	viewControlFrame.add(viewControlBox);

	viewControlBox.pack_start(controlSelectionBox);
	viewControlBox.pack_start(viewControlTable);
	viewControlBox.pack_start(rotationAxisFrame);

	rotationAxisFrame.add(rotationAxisVBox);
	rotationAxisVBox.pack_start(rotationAxisHBox);

	rotationAxisHBox.pack_start(radioAxisX);
	rotationAxisHBox.pack_start(radioAxisY);
	rotationAxisHBox.pack_start(radioAxisZ);

	Gtk::RadioButton::Group axisRadioGroup = radioAxisX.get_group();
	radioAxisY.set_group(axisRadioGroup);
	radioAxisZ.set_group(axisRadioGroup);

	radioAxisZ.set_active();

	controlSelectionBox.pack_start(radioControlViewport);
	controlSelectionBox.pack_start(radioControlObjects);

	Gtk::RadioButton::Group controlRadioGroup = radioControlViewport.get_group();
	radioControlObjects.set_group(controlRadioGroup);

	entryControlConstant.signal_changed().connect(sigc::mem_fun(*this,&MainWindow::controlConstantChanged));

	Gtk::Stock::lookup( Gtk::Stock::GO_UP, Gtk::ICON_SIZE_BUTTON, btUpImg);
	btUp.add(btUpImg); btUp.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onUpClicked));
	Gtk::Stock::lookup( Gtk::Stock::GO_DOWN, Gtk::ICON_SIZE_SMALL_TOOLBAR, btDownImg);
	btDown.add(btDownImg); btDown.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onDownClicked));
	Gtk::Stock::lookup( Gtk::Stock::GO_FORWARD, Gtk::ICON_SIZE_SMALL_TOOLBAR, btRightImg);
	btRight.add(btRightImg); btRight.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onRightClicked));
	Gtk::Stock::lookup( Gtk::Stock::GO_BACK, Gtk::ICON_SIZE_SMALL_TOOLBAR, btLeftImg);
	btLeft.add(btLeftImg); btLeft.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onLeftClicked));
	Gtk::Stock::lookup( Gtk::Stock::ZOOM_IN, Gtk::ICON_SIZE_SMALL_TOOLBAR, btZoomInImg);
	btZoomIn.add(btZoomInImg); btZoomIn.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onZoomInClicked));
	Gtk::Stock::lookup( Gtk::Stock::ZOOM_OUT, Gtk::ICON_SIZE_SMALL_TOOLBAR, btZoomOutImg);
	btZoomOut.add(btZoomOutImg); btZoomOut.signal_clicked().connect(sigc::mem_fun(*this, &MainWindow::onZoomOutClicked));
	Gtk::Stock::lookup( Gtk::Stock::REDO, Gtk::ICON_SIZE_SMALL_TOOLBAR, btRotateCWImg);
	btRotateCW.add(btRotateCWImg); btRotateCW.signal_clicked().connect(sigc::mem_fun(*this,&MainWindow::onRotateCWClicked));
	Gtk::Stock::lookup( Gtk::Stock::UNDO, Gtk::ICON_SIZE_SMALL_TOOLBAR, btRotateCCWImg);
	btRotateCCW.add(btRotateCCWImg); btRotateCCW.signal_clicked().connect(sigc::mem_fun(*this,&MainWindow::onRotateCCWClicked));

	viewControlTable.attach(btUp,1,2,0,1);
	viewControlTable.attach(btZoomIn,2,3,0,1);
	viewControlTable.attach(btZoomOut,0,1,0,1);
	viewControlTable.attach(btLeft,0,1,1,2);
	viewControlTable.attach(entryControlConstant,1,2,1,2);
	viewControlTable.attach(btRight,2,3,1,2);
	viewControlTable.attach(btRotateCW,2,3,2,3);
	viewControlTable.attach(btRotateCCW,0,1,2,3);
	viewControlTable.attach(btDown,1,2,2,3);

	rotationAxisVBox.pack_start(referenceHBox);
	referenceHBox.pack_start(radioReferenceCenter);
	referenceHBox.pack_start(radioReferenceOther);

	Gtk::RadioButton::Group referenceRadioGroup = radioReferenceCenter.get_group();
	radioReferenceOther.set_group(referenceRadioGroup);

	radioReferenceOther.signal_clicked().connect(sigc::mem_fun(*this,&MainWindow::onChangeReferenceClicked));
}

void MainWindow::controlConstantChanged(){
	std::stringstream strBuffer;
	strBuffer << entryControlConstant.get_text();
	strBuffer >> controlConstant;
}

void MainWindow::onUpClicked(){
	if(radioControlViewport.get_active())
		display.translateWindow(0,controlConstant,0);
	else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
			display.translate( row[elementColumn.elementName], 0, controlConstant);
		}catch(std::exception& e) {}
	}


	viewport.redraw();
}

void MainWindow::onDownClicked(){
	if(radioControlViewport.get_active())
		display.translateWindow(0,-controlConstant,0);
	else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
			display.translate( row[elementColumn.elementName], 0, -controlConstant);
		}catch(std::exception& e) {}
	}

	viewport.redraw();
}

void MainWindow::onLeftClicked(){
	if(radioControlViewport.get_active())
		display.translateWindow(-controlConstant,0,0);
	else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
			display.translate( row[elementColumn.elementName], -controlConstant, 0);
		}catch(std::exception& e) {}
	}

	viewport.redraw();
}

void MainWindow::onRightClicked(){
	if(radioControlViewport.get_active())
		display.translateWindow(controlConstant,0,0);
	else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
			display.translate( row[elementColumn.elementName], controlConstant, 0);
		}catch(std::exception& e) {}
	}

	viewport.redraw();
}

void MainWindow::onZoomInClicked(){
	if(radioControlViewport.get_active()){
		double scaleRatio = 1 - controlConstant/100;
		display.scaleWindow(scaleRatio, scaleRatio, scaleRatio);
	}else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
			double scaleRatio = 1 + controlConstant/100;
			display.scale(row[elementColumn.elementName], scaleRatio, scaleRatio, scaleRatio);
		}catch(std::exception& e) {}
	}

	viewport.redraw();
}

void MainWindow::onZoomOutClicked(){
	if(radioControlViewport.get_active()){
		double scaleRatio = 1 + controlConstant/100;
		display.scaleWindow(scaleRatio, scaleRatio, scaleRatio);
	}else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
			double scaleRatio = 1 - controlConstant/100;
			display.scale(row[elementColumn.elementName], scaleRatio, scaleRatio, scaleRatio);
		}catch(std::exception& e) {}
	}

	viewport.redraw();
}

MatrixFactory::Axis MainWindow::getRotationAxis(){
	if(radioAxisX.get_active())
		return MatrixFactory::X;
	if(radioAxisY.get_active())
		return MatrixFactory::Y;

	return MatrixFactory::Z;
}

void MainWindow::onRotateCWClicked(){
	if(radioControlViewport.get_active()){
		display.rotateWindow(getRotationAxis(), -controlConstant);
	}else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());

			if(radioReferenceCenter.get_active())
				display.rotate(row[elementColumn.elementName], getRotationAxis(), -controlConstant);
			else
				display.rotate(row[elementColumn.elementName], getRotationAxis(), -controlConstant, transformationReference );
		}catch(std::exception& e) {}
	}

	viewport.redraw();
}

void MainWindow::onRotateCCWClicked(){
	if(radioControlViewport.get_active()){
		display.rotateWindow(getRotationAxis(), controlConstant);
	}else{
		try{
			Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());

			if(radioReferenceCenter.get_active())
				display.rotate(row[elementColumn.elementName], getRotationAxis(), controlConstant);
			else
				display.rotate(row[elementColumn.elementName], getRotationAxis(), controlConstant, transformationReference );
		}catch(std::exception& e) {}
	}

	viewport.redraw();

}

void MainWindow::onChangeReferenceClicked(){
	if(radioReferenceCenter.get_active())
		return;

	CreatePointWindow createPointWindow(*this,false);
	int result = createPointWindow.run();

	if(result == Gtk::RESPONSE_OK){
		try{
			Point *point = createPointWindow.getPoint();
			transformationReference = point->getCoordinate();
			delete point;
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}
}

void MainWindow::addMenuToolbar(){
	  actionGroup = Gtk::ActionGroup::create();

	  //File - >New sub menu:
	  actionGroup->add(Gtk::Action::create("FileNewPoint",
	             "Point...", "Create a new point"),
	          sigc::mem_fun(*this, &MainWindow::onCreatePointClicked));

	  actionGroup->add(Gtk::Action::create("FileNewPolyline",
	              "Polyline...", "Create a new polyline"),
	          sigc::mem_fun(*this, &MainWindow::onCreatePolylineClicked));

	  actionGroup->add(Gtk::Action::create("FileNewPolygon",
	              "Polygon...", "Create a new polygon"),
	          sigc::mem_fun(*this, &MainWindow::onCreatePolygonClicked));

	  actionGroup->add(Gtk::Action::create("FileNewBezier",
				  "Bezier Curve...", "Create a new bezier curve"),
			sigc::mem_fun(*this, &MainWindow::onCreateBezierClicked));

	  actionGroup->add(Gtk::Action::create("FileNewBSpline",
	  				  "BSpline Curve...", "Create a new BSpline curve"),
	  			sigc::mem_fun(*this, &MainWindow::onCreateBSplineClicked));

	  actionGroup->add(Gtk::Action::create("FileNewBSurface",
					  "Bicubic Surface...", "Create a new bicubic surface"),
				sigc::mem_fun(*this, &MainWindow::onCreateBSurfaceClicked));

	  //File -> Save sub menu:
	  actionGroup->add(Gtk::Action::create("FileSaveObject", " Object..."),
	  			  sigc::mem_fun(*this, &MainWindow::on_save_object_clicked));

	  actionGroup->add(Gtk::Action::create("FileSaveScene", " Scene..."),
				  sigc::mem_fun(*this, &MainWindow::on_save_scene_clicked));

	  //File -> Open sub menu:
	  actionGroup->add(Gtk::Action::create("FileOpenObject", " Object..."),
	 	  			  sigc::mem_fun(*this, &MainWindow::on_open_object_clicked));

	  actionGroup->add(Gtk::Action::create("FileOpenScene", " Scene..."),
	 				  sigc::mem_fun(*this, &MainWindow::on_open_scene_clicked));

	  //File menu:
	  actionGroup->add(Gtk::Action::create("FileMenu", "File"));

	  //Sub-menu.
	  actionGroup->add(Gtk::Action::create("FileNew", Gtk::Stock::NEW, "New" ));
	  actionGroup->add(Gtk::Action::create("FileSave", Gtk::Stock::SAVE, "Save" ));
	  actionGroup->add(Gtk::Action::create("FileOpen", Gtk::Stock::OPEN, "Open"));

	  actionGroup->add(Gtk::Action::create("FileQuit", Gtk::Stock::QUIT), sigc::mem_fun(*this, &MainWindow::hide));

	  uiManager = Gtk::UIManager::create();
	  uiManager->insert_action_group(actionGroup);

	  add_accel_group(uiManager->get_accel_group());

	  //Layout the actions in a menubar and toolbar:
	  Glib::ustring ui_info =
	        "<ui>"
	        "  <menubar name='MenuBar'>"
	        "    <menu action='FileMenu'>"
	        "      <menu action='FileNew'>"
	        "        <menuitem action='FileNewPoint'/>"
	        "        <menuitem action='FileNewPolyline'/>"
	        "        <menuitem action='FileNewPolygon'/>"
			"		 <menuitem action='FileNewBezier' />"
			"		 <menuitem action='FileNewBSpline' />"
			"		 <menuitem action='FileNewBSurface' />"
	        "      </menu>"
			"      <menu action='FileSave'>"
			" 	     <menuitem action = 'FileSaveObject' />"
			"		 <menuitem action = 'FileSaveScene'  />"
			"      </menu>"
			"      <menu action='FileOpen'>"
			" 	     <menuitem action = 'FileOpenObject' />"
			"		 <menuitem action = 'FileOpenScene'  />"
			"      </menu>"
	        "      <separator/>"
	        "      <menuitem action='FileQuit'/>"
	        "    </menu>"
	        "  </menubar>"
	        "</ui>";

	  uiManager->add_ui_from_string(ui_info);

	  //Get the menubar and toolbar widgets, and add them to a container widget:
	  Gtk::Widget* pMenubar = uiManager->get_widget("/MenuBar");
	  if(pMenubar)
	    mainVBox.pack_start(*pMenubar, Gtk::PACK_SHRINK);
}

void MainWindow::onCreatePointClicked(){
	CreatePointWindow createPointWindow(*this);
	int result = createPointWindow.run();

	if(result == Gtk::RESPONSE_OK){
		try{
			display.add(createPointWindow.getPoint());
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}

	refresh();
}

void MainWindow::onCreatePolylineClicked(){
	CreatePolylineWindow polylineWindow(*this);
	int response = polylineWindow.run();

	if(response == Gtk::RESPONSE_OK){
		try{
			display.add(polylineWindow.getElement());
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}

	refresh();
}

void MainWindow::onCreatePolygonClicked(){
	CreatePolygonWindow polygonWindow(*this);
	int result = polygonWindow.run();

	if(result == Gtk::RESPONSE_OK){
		try{
			display.add(polygonWindow.getElement());
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}

	refresh();
}

void MainWindow::onCreateBezierClicked(){
	CreateBezierWindow bezierWindow(*this);
	int result = bezierWindow.run();

	if(result == Gtk::RESPONSE_OK){
		try{
			display.add(bezierWindow.getElement());
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}

	refresh();
}

void MainWindow::onCreateBSplineClicked(){
	CreateBSplineWindow bsplineWindow(*this);
	int result = bsplineWindow.run();

	if(result == Gtk::RESPONSE_OK){
		try{
			display.add(bsplineWindow.getElement());
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}

	refresh();
}

void MainWindow::onCreateBSurfaceClicked(){
	CreateBSurfaceWindow bsurfaceWindow(*this);
	int result = bsurfaceWindow.run();

	if(result == Gtk::RESPONSE_OK){
		try{
			display.add(bsurfaceWindow.getElement());
		}catch(std::exception& e) {
			ErrorDialog errorDialog(*this,e);
			errorDialog.run();
		}
	}

	refresh();
}

void MainWindow::refresh(){
	refreshElementsList();
	viewport.redraw();
}

void MainWindow::on_save_object_clicked(){
	try{
		Gtk::TreeModel::Row row = *(elementsView.get_selection()->get_selected());
		const GeometricElement* element = display.getElement(row[elementColumn.elementName]);

		Gtk::FileChooserDialog dialog("Choose destination file", Gtk::FILE_CHOOSER_ACTION_SAVE);

		dialog.set_transient_for(*this);
		//Add response buttons the the dialog:
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button("Select", Gtk::RESPONSE_OK);
		int result = dialog.run();
		if(result == Gtk::RESPONSE_OK)
			element->save(dialog.get_filename());

	}catch(std::exception& e){
		ErrorDialog errorDialog(*this,e);
		errorDialog.run();
	}
}

void MainWindow::on_save_scene_clicked(){
	try{
		Gtk::FileChooserDialog dialog("Choose destination file", Gtk::FILE_CHOOSER_ACTION_SAVE);

		dialog.set_transient_for(*this);
		//Add response buttons the the dialog:
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button("Select", Gtk::RESPONSE_OK);
		int result = dialog.run();
		if(result == Gtk::RESPONSE_OK)
			display.save(dialog.get_filename());

	}catch(std::exception& e){
		ErrorDialog errorDialog(*this,e);
		errorDialog.run();
	}
}

void MainWindow::on_open_object_clicked(){
	try{
		Gtk::FileChooserDialog dialog("Choose file", Gtk::FILE_CHOOSER_ACTION_OPEN);
		dialog.set_transient_for(*this);
		//Add response buttons the the dialog:
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button("Select", Gtk::RESPONSE_OK);
		int result = dialog.run();
		if(result == Gtk::RESPONSE_OK)
			display.add(GeometricElement::load(dialog.get_filename()));
	}catch(std::exception& e){
		ErrorDialog errorDialog(*this,e);
		errorDialog.run();
	}

	refresh();
}

void MainWindow::on_open_scene_clicked(){
	try{
		Gtk::FileChooserDialog dialog("Choose file", Gtk::FILE_CHOOSER_ACTION_OPEN);
		dialog.set_transient_for(*this);
		//Add response buttons the the dialog:
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button("Select", Gtk::RESPONSE_OK);
		int result = dialog.run();
		if(result == Gtk::RESPONSE_OK){
			changeScene(Display::load(dialog.get_filename()));
		}
	}catch(std::exception& e){
		ErrorDialog errorDialog(*this,e);
		errorDialog.run();
	}

	refresh();
}

void MainWindow::changeScene(Display* newScene){
	display = *newScene;

	refresh();
}

#include <iostream>
void MainWindow::createTestElements(){
	Line *xAxis = new Line(Coordinate(0,1000,0), Coordinate(0,-1000,0), "xAxis", DrawCfg(Color("#FF0000")));
	Line *yAxis = new Line(Coordinate(1000,0,0), Coordinate(-1000,0,0), "yAxis", DrawCfg(Color("#00FF00")));
	Line *zAxis = new Line(Coordinate(0,0,1000), Coordinate(0,0,-1000), "zAxis", DrawCfg(Color("#0000FF")));

	display.add(xAxis); display.add(yAxis); display.add(zAxis);

	for(int i = 0; i < 120; i+=3){
		CoordinateList polygon1Points;
		polygon1Points.push_back(Coordinate(10,10,-i*2));
		polygon1Points.push_back(Coordinate(10,310,-i*2));
		polygon1Points.push_back(Coordinate(310,310,-i*2));
		polygon1Points.push_back(Coordinate(310,10,-i*2));

		std::stringstream buffer;
		buffer << "p" << i;
		Polygon *polygon = new Polygon(polygon1Points, buffer.str(), false);
		polygon->rotate(MatrixFactory::Z,i);

		display.add(polygon);
	}

//	CoordinateList polygon2Points;
//	polygon2Points.push_back(Coordinate(60,60,1));
//	polygon2Points.push_back(Coordinate(60,360,1));
//	polygon2Points.push_back(Coordinate(360,360,1));
//	polygon2Points.push_back(Coordinate(360,60,1));
//	display.add(new Polygon(polygon2Points, "polygon2", true, DrawCfg(Color("#000000"))));

	CoordinateList bezier1;

	bezier1.push_back(Coordinate(200,430,1));
	bezier1.push_back(Coordinate(230,230,1));
	bezier1.push_back(Coordinate(400,630,1));
	bezier1.push_back(Coordinate(480,430,1));
	bezier1.push_back(Coordinate(400,430,1));
	bezier1.push_back(Coordinate(520,230,1));
	bezier1.push_back(Coordinate(200,430,1));

	display.add(new Bezier(bezier1,"bezier1"));

	CoordinateList polygon3Coordinates;
	polygon3Coordinates.push_back(Coordinate(10,300,-50));
	polygon3Coordinates.push_back(Coordinate(-300,400,-50));
	polygon3Coordinates.push_back(Coordinate(-300,100,-50));
	polygon3Coordinates.push_back(Coordinate(10,200,-50));
	polygon3Coordinates.push_back(Coordinate(-100,250,-50));
	display.add(new Polygon(polygon3Coordinates,"polygon3",true,DrawCfg(Color("#FF7700"))));


	CoordinateList element3dCoordinates;
	element3dCoordinates.push_back(Coordinate(0,0,0));
	element3dCoordinates.push_back(Coordinate(300,0,0));
	element3dCoordinates.push_back(Coordinate(300,300,0));
	element3dCoordinates.push_back(Coordinate(0,300,0));
	element3dCoordinates.push_back(Coordinate(0,0,300));
	element3dCoordinates.push_back(Coordinate(300,0,300));
	element3dCoordinates.push_back(Coordinate(300,300,300));
	element3dCoordinates.push_back(Coordinate(0,300,300));
	EdgeList element3dEdges;
	element3dEdges.push_back(std::pair<unsigned,unsigned>(0,1));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(1,2));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(2,3));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(3,0));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(4,5));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(5,6));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(6,7));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(7,4));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(0,4));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(1,5));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(2,6));
	element3dEdges.push_back(std::pair<unsigned,unsigned>(3,7));

	display.add(new Element3D(element3dCoordinates,element3dEdges,"element3d"));

	CoordinateList controlBicubicSurface;
	controlBicubicSurface.push_back(Coordinate(20,20,5));
	controlBicubicSurface.push_back(Coordinate(120,200,5));
	controlBicubicSurface.push_back(Coordinate(200,20,5));
	controlBicubicSurface.push_back(Coordinate(80,20,5));
	controlBicubicSurface.push_back(Coordinate(20,50,50));
	controlBicubicSurface.push_back(Coordinate(120,130,50));
	controlBicubicSurface.push_back(Coordinate(200,120,50));
	controlBicubicSurface.push_back(Coordinate(80,120,50));
	controlBicubicSurface.push_back(Coordinate(20,20,80));
	controlBicubicSurface.push_back(Coordinate(120,200,80));
	controlBicubicSurface.push_back(Coordinate(200,20,80));
	controlBicubicSurface.push_back(Coordinate(80,20,80));
	controlBicubicSurface.push_back(Coordinate(20,0,120));
	controlBicubicSurface.push_back(Coordinate(120,180,120));
	controlBicubicSurface.push_back(Coordinate(200,0,120));
	controlBicubicSurface.push_back(Coordinate(80,0,120));

	BicubicSurface *bicubicSurface = new BicubicSurface(controlBicubicSurface,"surface");
	display.add(bicubicSurface);
}

MainWindow::~MainWindow() {
}
