#ifndef MATRIXTESTS_H_
#define MATRIXTESTS_H_

#include <cpptest.h>
#include "Matrix.h"
#include "MatrixFactory.h"

class MatrixTests : public Test::Suite {
public:
	MatrixTests(){
		TEST_ADD(MatrixTests::equality_between_matrices);
		TEST_ADD(MatrixTests::empty_matrix_throws);
		TEST_ADD(MatrixTests::matrix_string_representations);
		TEST_ADD(MatrixTests::getting_elements);
		TEST_ADD(MatrixTests::tests_inserting_rows);
		TEST_ADD(MatrixTests::multiplying_matrices);
	}

	void equality_between_matrices(){
		Matrix<int> m1(3,1);
		m1(0,0) = 1;
		m1(1,0) = 2;
		m1(2,0) = 3;

		Matrix<int> m2(3,1);
		m2(0,0) = 3; m2(1,0) = 2; m2(2,0) = 1;

		Matrix<int> m3(3,1);
		m3(0,0) = 1; m3(1,0) = 2; m3(2,0) = 3;

		Matrix<int> m4(4,1);
		m4(0,0) = 1; m4(1,0) = 2; m4(2,0) = 3; m4(3,0) = 4;

		TEST_ASSERT(m1 != m2);
		TEST_ASSERT(m1 == m3);
		TEST_ASSERT(m1 != m4);

		Matrix<int> m6(2,4);
		for(unsigned i = 0; i < 4; i++)
			for(unsigned j = 0; j < 2; j++)
				m6(j,i) = i+j;
		Matrix<int> m7(m6);

		TEST_ASSERT(m7 == m6);
		TEST_ASSERT(m1 == Matrix<int>(m1));

		TEST_ASSERT(m7(1,2) == 3);
	}

	void matrix_string_representations(){
		Matrix<int> m4(4,1);
		m4(0,0) = 1; m4(1,0) = 2; m4(2,0) = 3; m4(3,0) = 4;
		TEST_ASSERT(m4.toString() == "| 1 |\n| 2 |\n| 3 |\n| 4 |\n");

		Matrix<int> m2(3,1);
		m2(0,0) = 3; m2(1,0)= 2; m2(2,0) = 1;
		TEST_ASSERT(m2.toString() == "| 3 |\n| 2 |\n| 1 |\n");

		Matrix<int> m5(3,3);
		for(unsigned i = 0; i < 3; i++)
			for(unsigned j = 0; j < 3; j++)
				m5(j,i) = i+j;

		TEST_ASSERT(m5.toString() == "| 0 1 2 |\n| 1 2 3 |\n| 2 3 4 |\n");

		Matrix<int> m6(4,2);
		for(unsigned i = 0; i < 4; i++)
			for(unsigned j = 0; j < 2; j++)
				m6(i,j) = i+j;

		TEST_ASSERT(m6.toString() == "| 0 1 |\n| 1 2 |\n| 2 3 |\n| 3 4 |\n");
	}

	void empty_matrix_throws(){
		TEST_THROWS_ANYTHING(Matrix<int>(0,1));
		TEST_THROWS_ANYTHING(Matrix<int>(10,0));
	}

	void getting_elements(){
		Matrix<int> m1(3,1);
		m1(0,0) = 1; m1(1,0) = 2; m1(2,0) = 3;
		TEST_ASSERT(m1(0,0) == 1);
		TEST_ASSERT(m1(1,0) == 2);
		TEST_ASSERT(m1(2,0) == 3);

		Matrix<int> m6(6,6);
		for(unsigned i = 0; i < 6; i++)
			for(unsigned j = 0; j < 6; j++)
				m6(i,j) = i+j;

		for(int i = 0; i < 6; i++)
			for(int j = 0; j < 6; j++)
				TEST_ASSERT(m6(i,j) == i+j);

	}

	void tests_inserting_rows(){
		Matrix<int> m1(3,1);
		m1(0,0) = 1; m1(1,0) = 2; m1(2,0) = 3;

		Matrix<int> m2(3,3);
		m2(0,0) = 1; m2(0,1) = 2; m2(0,2) = 3;
		m2(1,0) = 2; m2(1,1) = 2; m2(1,2) = 1;
		m2(2,0) = 3; m2(2,1) = 1; m2(2,2) = 0;

		TEST_ASSERT(m1 != m2);

		m1.insertColumn(m2.getColumn(1));
		m1.insertColumn(m2.getColumn(2));

		TEST_ASSERT(m1 == m2);
	}

	void multiplying_matrices(){
		Matrix<int> m1(3,3);
		for(unsigned i = 0; i < 3; i++)
			for(unsigned j = 0; j < 3; j++)
				m1(i,j) = i+j;

		Matrix<int> m2(3,3);
		m2(0,0) = 5; m2(0,1) = 8; m2(0,2) = 11;
		m2(1,0) = 8; m2(1,1) = 14; m2(1,2) = 20;
		m2(2,0) = 11; m2(2,1) = 20; m2(2,2) = 29;

		TEST_ASSERT(*(m1 * m1) == m2);

		Matrix<int> m3(1,2);
		m3(0,0) = 2;	m3(0,1) = 3;

		Matrix<int> m4(2,2);
		m4(0,0) = 4; m4(0,1) = 6;
		m4(1,0) = 0; m4(1,1) = 0;

		Matrix<int> m5(1,2);
		m5(0,0) = 8;	m5(0,1)= 12;

		TEST_ASSERT(*(m3 * m4) == m5);

		TEST_THROWS_ANYTHING(m1 * m3);

		Matrix<int> m6(2,3);
		m6(0,0) = 1; m6(0,1) = 0; m6(0,2) = 2;
		m6(1,0) = -1; m6(1,1) = 3; m6(1,2) = 1;

		Matrix<int> m7(3,2);
		m7(0,0) = 3; m7(0,1) = 1;
		m7(1,0) = 2; m7(1,1) = 1;
		m7(2,0) = 1; m7(2,1) = 0;

		Matrix<int> m8(2,2);
		m8(0,0) = 5; m8(0,1) = 1;
		m8(1,0) = 4; m8(1,1) = 2;

		TEST_ASSERT( *(m6 * m7) == m8);
		TEST_ASSERT( m6 * m7 != m7 * m6);
	}

	virtual ~MatrixTests() {}
};

#endif /* MATRIXTESTS_H_ */
