#ifndef XMLTESTS_H_
#define XMLTESTS_H_

#include <cpptest.h>
#include "Element3D.h"
#include  "BSpline.h"
#include "Bezier.h"
#include <iostream>
#include "Display.h"
#include "BicubicSurface.h"

class XmlTests : public Test::Suite{

	#define GS_TESTS_XML_PREFIX "src/tests/xml_test/"

public:
	XmlTests(){
		TEST_ADD(XmlTests::create_point);
		TEST_ADD(XmlTests::create_line);
		TEST_ADD(XmlTests::create_polyline);
		TEST_ADD(XmlTests::create_polygon);
		TEST_ADD(XmlTests::create_bezier);
		TEST_ADD(XmlTests::create_bspline);
		TEST_ADD(XmlTests::create_elemente3d);
		TEST_ADD(XmlTests::load_point);
		TEST_ADD(XmlTests::load_line);
		TEST_ADD(XmlTests::load_polyline);
		TEST_ADD(XmlTests::load_polygon);
		TEST_ADD(XmlTests::load_element3d);
		TEST_ADD(XmlTests::load_bspline);
		TEST_ADD(XmlTests::load_bezier);
		TEST_ADD(XmlTests::load_scene);
		TEST_ADD(XmlTests::create_surface);
		TEST_ADD(XmlTests::load_surface);
	}

	void create_point(){
		Point point1(Coordinate(10,20,30),"pointtt",DrawCfg(Color("#FF0000")));

		TEST_THROWS_NOTHING(point1.save(GS_TESTS_XML_PREFIX"point1.xml"));
	}

	void create_line(){
		Line line1(Coordinate(10,20,30),Coordinate(30.5,-300.4223,-10.11),"lineee",DrawCfg(Color("#FFFF77")));

		TEST_THROWS_NOTHING(line1.save(GS_TESTS_XML_PREFIX"line1.xml"));
	}

	void create_polyline(){
		CoordinateList polyline1coordinates;
		polyline1coordinates.push_back(Coordinate(10,300,-50));
		polyline1coordinates.push_back(Coordinate(-300,400,-50));
		polyline1coordinates.push_back(Coordinate(-300,100,-50));
		polyline1coordinates.push_back(Coordinate(10,200,-50));
		polyline1coordinates.push_back(Coordinate(-100,250,-50));
		Polyline polyline1(polyline1coordinates,"polilinhaa",DrawCfg(Color("#FF7700")));

		TEST_THROWS_NOTHING(polyline1.save(GS_TESTS_XML_PREFIX"polyline1.xml"));
	}

	void create_polygon(){
		CoordinateList polygonCoordinates;
		polygonCoordinates.push_back(Coordinate(10,300,-50));
		polygonCoordinates.push_back(Coordinate(-300,400,-50));
		polygonCoordinates.push_back(Coordinate(-300,100,-50));
		polygonCoordinates.push_back(Coordinate(10,200,-50));
		polygonCoordinates.push_back(Coordinate(-100,250,-50));

		Polygon polygon1(polygonCoordinates,"poligono1",true,DrawCfg(Color("#FF11FF")));
		TEST_THROWS_NOTHING(polygon1.save(GS_TESTS_XML_PREFIX"polygon1.xml"));

		Polygon polygon2(polygonCoordinates,"poligono2",false,DrawCfg(Color("#FFFFFF")));
		TEST_THROWS_NOTHING(polygon2.save(GS_TESTS_XML_PREFIX"polygon2.xml"));
	}

	void create_bezier(){
		CoordinateList bezier1;

		bezier1.push_back(Coordinate(200,430,1));
		bezier1.push_back(Coordinate(230,230,1));
		bezier1.push_back(Coordinate(400,630,1));
		bezier1.push_back(Coordinate(480,430,1));
		bezier1.push_back(Coordinate(400,430,1));
		bezier1.push_back(Coordinate(520,230,1));
		bezier1.push_back(Coordinate(200,430,1));

		Bezier bezier(bezier1,"bezierrrrrr");

		TEST_THROWS_NOTHING(bezier.save(GS_TESTS_XML_PREFIX"bezier1.xml"));
	}

	void create_bspline(){
		CoordinateList controlBs;
		controlBs.push_back(Coordinate(20,100,1));
		controlBs.push_back(Coordinate(120,200,1));
		controlBs.push_back(Coordinate(220,0,1));
		controlBs.push_back(Coordinate(320,100,1));
		controlBs.push_back(Coordinate(220,200,1));
		controlBs.push_back(Coordinate(120,0,1));
		controlBs.push_back(Coordinate(20,100,1));
		controlBs.push_back(Coordinate(120,200,1));
		controlBs.push_back(Coordinate(220,0,1));

		BSpline bspline(controlBs,"bsplineeeeee",100,DrawCfg(Color("#FFFFFF")));
		TEST_THROWS_NOTHING(bspline.save(GS_TESTS_XML_PREFIX"bspline1.xml"));
	}

	void create_elemente3d(){
		CoordinateList coordinates;
		coordinates.push_back(Coordinate(0,0,0));
		coordinates.push_back(Coordinate(300,0,0));
		coordinates.push_back(Coordinate(300,300,0));
		coordinates.push_back(Coordinate(0,300,0));
		coordinates.push_back(Coordinate(0,0,300));
		coordinates.push_back(Coordinate(300,0,300));
		coordinates.push_back(Coordinate(300,300,300));
		coordinates.push_back(Coordinate(0,300,300));

		EdgeList edges;
		edges.push_back(std::pair<unsigned,unsigned>(0,1));
		edges.push_back(std::pair<unsigned,unsigned>(1,2));
		edges.push_back(std::pair<unsigned,unsigned>(2,3));
		edges.push_back(std::pair<unsigned,unsigned>(3,0));
		edges.push_back(std::pair<unsigned,unsigned>(4,5));
		edges.push_back(std::pair<unsigned,unsigned>(5,6));
		edges.push_back(std::pair<unsigned,unsigned>(6,7));
		edges.push_back(std::pair<unsigned,unsigned>(7,4));
		edges.push_back(std::pair<unsigned,unsigned>(0,4));
		edges.push_back(std::pair<unsigned,unsigned>(1,5));
		edges.push_back(std::pair<unsigned,unsigned>(2,6));
		edges.push_back(std::pair<unsigned,unsigned>(3,7));

		Element3D element3d(coordinates,edges,"element3dddd",DrawCfg(Color("#FF0000")));
		TEST_THROWS_NOTHING(element3d.save(GS_TESTS_XML_PREFIX"element3d.xml"));
	}



	void load_point(){
		try {
			GeometricElement *pt = GeometricElement::load(GS_TESTS_XML_PREFIX"point1.xml");
			std::cout << pt->toString() << std::endl;
		} catch (std::exception& e) {
			std:: cout << e.what() << std::endl;
		}


	}

	void load_line(){
		GeometricElement *line = GeometricElement::load(GS_TESTS_XML_PREFIX"line1.xml");
		std::cout << line->toString() << std::endl;
	}

	void load_polyline(){
		GeometricElement *polyline = GeometricElement::load(GS_TESTS_XML_PREFIX"polyline1.xml");
		std::cout << polyline->toString() << std::endl;
	}

	void load_polygon(){
		GeometricElement *polygon = GeometricElement::load(GS_TESTS_XML_PREFIX"polygon1.xml");
		std::cout << polygon->toString() << polygon->is(GeometricElement::FILLED_POLYGON) <<  std::endl;

		GeometricElement *polygon2 = GeometricElement::load(GS_TESTS_XML_PREFIX"polygon2.xml");
		std::cout << polygon2->toString()<< polygon2->is(GeometricElement::FILLED_POLYGON)  << std::endl ;
	}

	void load_element3d(){
		GeometricElement *element3d = GeometricElement::load(GS_TESTS_XML_PREFIX"element3d.xml");
		std::cout << element3d->toString();
	}

	void load_bspline(){
		GeometricElement *bspline = GeometricElement::load(GS_TESTS_XML_PREFIX"bspline1.xml");
		std::cout << bspline->toString();
	}

	void load_bezier(){
		GeometricElement *bezier = GeometricElement::load(GS_TESTS_XML_PREFIX"bezier1.xml");
		std::cout << bezier->toString();
	}

	void load_scene(){
		try {
			Display *display = Display::load("default_scene.xml");
			for(DisplayFile::Iterator i = display->begin(); i != display->end(); i++){
				std::cout << (*i)->toString() << std::endl;
			}
		} catch (std::exception& e) {
			std::cout << e.what() << std::endl;
		}
	}


	void create_surface(){
		CoordinateList controlBs;
		controlBs.push_back(Coordinate(20,20,5));
		controlBs.push_back(Coordinate(120,200,5));
		controlBs.push_back(Coordinate(200,20,5));
		controlBs.push_back(Coordinate(80,20,5));
		controlBs.push_back(Coordinate(20,50,50));
		controlBs.push_back(Coordinate(120,130,50));
		controlBs.push_back(Coordinate(200,120,50));
		controlBs.push_back(Coordinate(80,120,50));
		controlBs.push_back(Coordinate(20,20,80));
		controlBs.push_back(Coordinate(120,200,80));
		controlBs.push_back(Coordinate(200,20,80));
		controlBs.push_back(Coordinate(80,20,80));
		controlBs.push_back(Coordinate(20,0,120));
		controlBs.push_back(Coordinate(120,180,120));
		controlBs.push_back(Coordinate(200,0,120));
		controlBs.push_back(Coordinate(80,0,120));

		controlBs.push_back(Coordinate(2,2,5));
		controlBs.push_back(Coordinate(300,150,5));
		controlBs.push_back(Coordinate(200,20,5));
		controlBs.push_back(Coordinate(80,20,5));
		controlBs.push_back(Coordinate(20,50,50));
		controlBs.push_back(Coordinate(120,130,50));
		controlBs.push_back(Coordinate(200,120,50));
		controlBs.push_back(Coordinate(80,120,50));
		controlBs.push_back(Coordinate(20,20,80));
		controlBs.push_back(Coordinate(120,200,80));
		controlBs.push_back(Coordinate(200,20,80));
		controlBs.push_back(Coordinate(80,20,80));
		controlBs.push_back(Coordinate(20,0,120));
		controlBs.push_back(Coordinate(120,180,120));
		controlBs.push_back(Coordinate(200,0,120));
		controlBs.push_back(Coordinate(100,0,120));

		BicubicSurface bs(controlBs,"surfaceee",150,DrawCfg(Color("#FF00FF")));
		std::cout << bs.toString() << std::endl;
		bs.save(GS_TESTS_XML_PREFIX"bicubicSurface1.xml");
	}

	void load_surface(){
		GeometricElement *surface = GeometricElement::load(GS_TESTS_XML_PREFIX"bicubicSurface1.xml");
		std::cout << surface->toString() << std::endl;
	}

	virtual ~XmlTests(){

	}
};

#endif /* XMLTESTS_H_ */
